@@ -9,6 +9,7 @@
 #          Mats van Es <mats.vanes@psych.ox.ac.uk>
 
 import argparse
+import matplotlib
 import matplotlib.pyplot as plt
 import os
 import sys
@@ -343,9 +344,14 @@ def write_dataset(dataset, outbase, run_id, overwrite=False):
         The saved fif file name
     """
 
-    fif_outname = outbase.format(
-        run_id=run_id.replace("_raw", ""), ftype="preproc_raw", fext="fif"
-    )
+    if "preproc_raw" in run_id:
+        fif_outname = outbase.format(
+            run_id=run_id.replace("_preproc_raw", ""), ftype="preproc_raw", fext="fif"
+        )
+    else:
+        fif_outname = outbase.format(
+            run_id=run_id.replace("_raw", ""), ftype="preproc_raw", fext="fif"
+        )
     if Path(fif_outname).exists() and not overwrite:
         raise ValueError(
             "{} already exists. Please delete or do use overwrite=True.".format(fif_outname)
@@ -370,13 +376,15 @@ def write_dataset(dataset, outbase, run_id, overwrite=False):
 
     return fif_outname
 
-def read_dataset(fif):
+def read_dataset(fif, preload=False):
     """Reads fif/npy/yml files associated with a dataset.
 
     Parameters
     ----------
     fif : str
         Path to raw fif file (can be preprocessed).
+    preload : bool
+        Should we load the raw fif data?
 
     Returns
     -------
@@ -386,7 +394,7 @@ def read_dataset(fif):
     print("Loading dataset:")
 
     print("Reading", fif)
-    raw = mne.io.read_raw_fif(fif)
+    raw = mne.io.read_raw_fif(fif, preload=preload)
 
     events = Path(fif.replace("preproc_raw.fif", "events.npy"))
     if events.exists():
@@ -701,6 +709,26 @@ def run_proc_chain(
         if outdir is not None:
             fif_outname = write_dataset(dataset, outbase, run_id, overwrite=overwrite)
 
+        # Generate report data
+        if gen_report:
+            # Switch to non-GUI plotting backend
+            mpl_backend = matplotlib.pyplot.get_backend()
+            matplotlib.use('Agg')
+
+            from ..report import gen_html_data, gen_html_page  # avoids circular import
+            logger.info("{0} : Generating Report".format(now))
+            report_data_dir = validate_outdir(reportdir / Path(fif_outname).stem)
+            gen_html_data(
+                dataset["raw"],
+                report_data_dir,
+                ica=dataset["ica"],
+                preproc_fif_filename=fif_outname,
+            )
+            gen_html_page(reportdir)
+
+            # Restore plotting context
+            matplotlib.use(mpl_backend)
+
     except Exception as e:
         # Preprocessing failed
 
@@ -740,19 +768,6 @@ def run_proc_chain(
     if fif_outname is not None:
         logger.info("Output file is {}".format(fif_outname))
 
-    # Generate report data
-    if gen_report:
-        from ..report import gen_html_data, gen_html_page  # avoids circular import
-        logger.info("{0} : Generating Report".format(now))
-        report_data_dir = validate_outdir(reportdir / Path(fif_outname).stem)
-        gen_html_data(
-            dataset["raw"],
-            report_data_dir,
-            ica=dataset["ica"],
-            preproc_fif_filename=fif_outname,
-            logger=logger,
-        )
-        gen_html_page(reportdir)
 
     if ret_dataset:
         return dataset
@@ -892,6 +907,7 @@ def run_proc_batch(
     else:
         proc_flags = [pool_func(*aa) for aa in args]
 
+    osl_logger.set_up(log_file=logfile, level=verbose, startup=False)
     logger.info(
         "Processed {0}/{1} files successfully".format(
             np.sum(proc_flags), len(proc_flags)
@@ -927,20 +943,24 @@ def main(argv=None):
         help="plain text file containing full paths to files to be processed",
     )
     parser.add_argument(
-        "outdir", type=str, help="Path to output directory to save data in"
+        "--outdir",
+        type=str,
+        default=None,
+        help="Path to output directory to save data in",
     )
     parser.add_argument(
-        "logsdir", type=str, help="Path to logs directory"
+        "--logsdir", type=str, default=None, help="Path to logs directory"
     )
     parser.add_argument(
-        "reportdir", type=str, help="Path to report directory"
+        "--reportdir", type=str, default=None, help="Path to report directory"
     )
     parser.add_argument(
-        "gen_report", type=bool, help="Should we generate a report?"
+        "--gen_report", type=bool, default=True, help="Should we generate a report?"
     )
     parser.add_argument(
         "--overwrite",
         action="store_true",
+        default=False,
         help="Overwrite previous output files if they're in the way",
     )
     parser.add_argument(
@@ -955,6 +975,11 @@ def main(argv=None):
         default="WARNING",
         help="Set the logging level for MNE functions",
     )
+    parser.add_argument(
+        "--strictrun",
+        action="store_true",
+        help="Will ask the user for confirmation before starting",
+    )
 
     parser.usage = parser.format_help()
     args = parser.parse_args(argv)
