@@ -1,5 +1,3 @@
-#!/usr/bin/env python
-
 """Reporting tool for preprocessing.
 
 """
@@ -78,7 +76,7 @@ def get_header_id(raw):
     return raw.filenames[0].split('/')[-1].strip('.fif')
 
 
-def gen_html_data(raw, outdir, ica=None, preproc_fif_filename=None, logger=None):
+def gen_html_data(raw, outdir, ica=None, preproc_fif_filename=None):
     """Generate HTML web-report for an MNE data object.
 
     Parameters
@@ -91,8 +89,6 @@ def gen_html_data(raw, outdir, ica=None, preproc_fif_filename=None, logger=None)
         ICA object.
     preproc_fif_filename : str
         Filename of file output by preprocessing
-    logger : logging.getLogger
-        Logger.
     """
 
     data = {}
@@ -115,7 +111,7 @@ def gen_html_data(raw, outdir, ica=None, preproc_fif_filename=None, logger=None)
     try:
         data['nhpi'] = len(raw.info['hpi_meas'][0]['hpi_coils'])
     except:
-        log_or_print("No HPI info in fif file", logger)
+        log_or_print("No HPI info in fif file")
 
     chtype = [channel_type(raw.info, c) for c in range(data['nchans'])]
     chs, chcounts = np.unique(chtype, return_counts=True)
@@ -149,12 +145,15 @@ def gen_html_data(raw, outdir, ica=None, preproc_fif_filename=None, logger=None)
 
     data['bad_seg'] = []
     for modality in ['grad', 'mag', 'eeg']:
+
         inds = [s.find(modality) > 0 for s in types]
         mod_dur = np.sum(durs[inds])
         pc = (mod_dur / full_dur) * 100
         s = 'Modality {0} - {1:.2f}/{2} seconds rejected     ({3:.2f}%)'
         if mod_dur > 0:
             data['bad_seg'].append(s.format(modality, mod_dur, full_dur, pc))
+            # For summary report:
+            data['bad_seg_pc_' + modality] = pc
 
     # Bad channels
     bad_chans = raw.info['bads']
@@ -162,6 +161,8 @@ def gen_html_data(raw, outdir, ica=None, preproc_fif_filename=None, logger=None)
         data['bad_chans'] = 'No bad channels.'
     else:
         data['bad_chans'] = 'Bad channels: {}'.format(', '.join(bad_chans))
+    # For summary report:
+    data['bad_chan_num'] = len(bad_chans)
 
     # Path to save plots
     savebase = str(outdir / '{0}.png')
@@ -182,7 +183,8 @@ def gen_html_data(raw, outdir, ica=None, preproc_fif_filename=None, logger=None)
 
     # Add ICA if it's been passed
     if ica is not None:
-        data['plt_ica'] = plot_bad_ica(raw, ica, savebase)
+        if len(ica.exclude) > 0:
+            data['plt_ica'] = plot_bad_ica(raw, ica, savebase)
 
     # Save data that will be used to create html page
     with open(outdir / 'data.pkl', 'wb') as outfile:
@@ -284,7 +286,11 @@ def gen_html_summary(reportdir):
     data["total"] = total
 
     # Create plots
+    os.makedirs(f"{reportdir}/summary", exist_ok=True)
+
     data["plt_config"] = subject_data[0]["plt_config"]
+    data["plt_summary_bad_segs"] = plot_summary_bad_segs(subject_data, reportdir)
+    data["plt_summary_bad_chans"] = plot_summary_bad_chans(subject_data, reportdir)
 
     # Create panel
     panel_template = load_template('raw_summary_panel')
@@ -375,8 +381,12 @@ def plot_channel_time_series(raw, savebase=None, exclude_bads=False):
         # ~28 reference axial grads if {picks: 'grad'}
 
         channel_types = {
-            'Axial Grads (chtype=''mag'')': mne.pick_types(raw.info, meg='mag', ref_meg=False, exclude=exclude),
-            'Ref Axial Grad (chtype=''ref_meg'')': mne.pick_types(raw.info, meg='grad', exclude=exclude),
+            'Axial Grads (chtype=mag)': mne.pick_types(
+                raw.info, meg='mag', ref_meg=False, exclude=exclude
+            ),
+            'Ref Axial Grad (chtype=ref_meg)': mne.pick_types(
+                raw.info, meg='grad', exclude=exclude
+            ),
             'EEG': mne.pick_types(raw.info, eeg=True),
             'CSD': mne.pick_types(raw.info, csd=True),
         }
@@ -806,59 +816,65 @@ def plot_bad_ica(raw, ica, savebase):
 
     # Create figure
     fig = plt.figure(figsize=(16, 5 * nbad), facecolor=[0.95] * 3)
-    axes = []
-    for i in np.arange(nbad):
-        lowerlimit = 0.1 + i / (nbad * 1.1)
-        multiplier = nbad * 1.3
+    if nbad == 0:
+        plt.subplot(111, frameon=False)
+        plt.text(0.5, 0.5, 'No Components Rejected', ha='center', va='center')
+        plt.xticks([])
+        plt.yticks([])
+    else:
+        axes = []
+        for i in np.arange(nbad):
+            lowerlimit = 0.1 + i / (nbad * 1.1)
+            multiplier = nbad * 1.3
 
-        # Create axis for subplot
-        # adapted from mne/viz/ica._create_properties_layout
-        if 'mag' in raw.get_channel_types() and 'grad' in raw.get_channel_types():
-            topomap1_layout = [0.08, lowerlimit + 0.5 / multiplier, (0.3-0.08)/2+0.08, 0.45 / multiplier]
-            topomap2_layout = [(0.3-0.08)/2+0.08, lowerlimit + 0.5 / multiplier, 0.3, 0.45 / multiplier]
-        else:
-            topomap1_layout = [0.08, lowerlimit + 0.5 / multiplier, 0.3, 0.45 / multiplier]
-        axes_params = (('topomap', topomap1_layout),
-                       ('image', [0.5, lowerlimit + 0.6 / multiplier, 0.45, 0.35 / multiplier]),
-                       ('erp', [0.5, lowerlimit + 0.5 / multiplier, 0.45, 0.1 / multiplier]),
-                       ('spectrum', [0.08, lowerlimit + 0.1 / multiplier, 0.32, 0.3 / multiplier]),
-                       ('variance', [0.5, lowerlimit + 0.025 / multiplier, 0.45, 0.25 / multiplier]))
-        axes += [[fig.add_axes(loc, label=name) for name, loc in axes_params]]
+            # Create axis for subplot
+            # adapted from mne/viz/ica._create_properties_layout
+            if 'mag' in raw.get_channel_types() and 'grad' in raw.get_channel_types():
+                topomap1_layout = [0.08, lowerlimit + 0.5 / multiplier, (0.3-0.08)/2+0.08, 0.45 / multiplier]
+                topomap2_layout = [(0.3-0.08)/2+0.08, lowerlimit + 0.5 / multiplier, 0.3, 0.45 / multiplier]
+            else:
+                topomap1_layout = [0.08, lowerlimit + 0.5 / multiplier, 0.3, 0.45 / multiplier]
+            axes_params = (('topomap', topomap1_layout),
+                           ('image', [0.5, lowerlimit + 0.6 / multiplier, 0.45, 0.35 / multiplier]),
+                           ('erp', [0.5, lowerlimit + 0.5 / multiplier, 0.45, 0.1 / multiplier]),
+                           ('spectrum', [0.08, lowerlimit + 0.1 / multiplier, 0.32, 0.3 / multiplier]),
+                           ('variance', [0.5, lowerlimit + 0.025 / multiplier, 0.45, 0.25 / multiplier]))
+            axes += [[fig.add_axes(loc, label=name) for name, loc in axes_params]]
 
-        ica.plot_properties(raw, picks=exclude_uniq[i], axes=axes[i], show=False, verbose=0)
+            ica.plot_properties(raw, picks=exclude_uniq[i], axes=axes[i], show=False, verbose=0)
 
-        # Add another topo if we're dealing with two sensor types
-        if 'mag' in raw.get_channel_types() and 'grad' in raw.get_channel_types():
-            ax2 = fig.add_axes(topomap2_layout, label='topomap2')
-            kind, dropped_indices, epochs_src, data = mne.viz.ica._prepare_data_ica_properties(
-                raw, ica, reject_by_annotation=True, reject='auto')
-            mne.viz.ica._plot_ica_topomap(ica, exclude_uniq[i], ch_type='grad',  axes=ax2, show=False)
+            # Add another topo if we're dealing with two sensor types
+            if 'mag' in raw.get_channel_types() and 'grad' in raw.get_channel_types():
+                ax2 = fig.add_axes(topomap2_layout, label='topomap2')
+                kind, dropped_indices, epochs_src, data = mne.viz.ica._prepare_data_ica_properties(
+                    raw, ica, reject_by_annotation=True, reject='auto')
+                mne.viz.ica._plot_ica_topomap(ica, exclude_uniq[i], ch_type='grad',  axes=ax2, show=False)
 
-        if np.any([x in ica.labels_.keys() for x in ica._ica_names]): # this is for the osl_plot_ica convention
-            title = "".join((ica._ica_names[exclude_uniq[i]]," - ", ica.labels_[ica._ica_names[exclude_uniq[i]]].upper()))
+            if np.any([x in ica.labels_.keys() for x in ica._ica_names]): # this is for the osl_plot_ica convention
+                title = "".join((ica._ica_names[exclude_uniq[i]]," - ", ica.labels_[ica._ica_names[exclude_uniq[i]]].upper()))
 
-        elif 'eog' in ica.labels_.keys() and 'ecg' in ica.labels_.keys(): # this is for the MNE automatic labelling convention
-            flag_eog = exclude_uniq[i] in ica.labels_['eog']
-            flag_ecg = exclude_uniq[i] in ica.labels_['ecg']
-            title = "".join((ica._ica_names[exclude_uniq[i]]," - ", flag_eog*'EOG', flag_ecg*flag_eog*'/', flag_ecg*'ECG'))
+            elif 'eog' in ica.labels_.keys() and 'ecg' in ica.labels_.keys(): # this is for the MNE automatic labelling convention
+                flag_eog = exclude_uniq[i] in ica.labels_['eog']
+                flag_ecg = exclude_uniq[i] in ica.labels_['ecg']
+                title = "".join((ica._ica_names[exclude_uniq[i]]," - ", flag_eog*'EOG', flag_ecg*flag_eog*'/', flag_ecg*'ECG'))
 
-        elif 'eog' in ica.labels_.keys():
-            flag_eog = exclude_uniq[i] in ica.labels_['eog']
-            title = "".join((ica._ica_names[exclude_uniq[i]]," - ", flag_eog*'EOG'))
+            elif 'eog' in ica.labels_.keys():
+                flag_eog = exclude_uniq[i] in ica.labels_['eog']
+                title = "".join((ica._ica_names[exclude_uniq[i]]," - ", flag_eog*'EOG'))
 
-        elif 'ecg' in ica.labels_.keys():
-            flag_ecg = exclude_uniq[i] in ica.labels_['ecg']
-            title = "".join((ica._ica_names[exclude_uniq[i]]," - ", flag_ecg*'ECG'))
+            elif 'ecg' in ica.labels_.keys():
+                flag_ecg = exclude_uniq[i] in ica.labels_['ecg']
+                title = "".join((ica._ica_names[exclude_uniq[i]]," - ", flag_ecg*'ECG'))
 
-        if 'mag' in raw.get_channel_types() and 'grad' in raw.get_channel_types():
-            title = title + '\n (mag)'
-            ax2.set_title(title.replace('mag', 'grad'))
+            if 'mag' in raw.get_channel_types() and 'grad' in raw.get_channel_types():
+                title = title + '\n (mag)'
+                ax2.set_title(title.replace('mag', 'grad'))
 
-        else: # this is for if there is nothing in ica.labels_
-            title = None
+            else: # this is for if there is nothing in ica.labels_
+                title = None
 
-        if title is not None:
-            axes[i][0].set_title(title, fontsize=12)
+            if title is not None:
+                axes[i][0].set_title(title, fontsize=12)
 
     if savebase is not None:
         figname = savebase.format('ica')
@@ -871,6 +887,83 @@ def plot_bad_ica(raw, ica, savebase):
     return filebase.format('ica')
 
 
+def plot_summary_bad_segs(subject_data, reportdir):
+    """Plot summary of bad channels over subjects.
+
+    Parameters
+    ----------
+    subject_data : list
+        list of data for each subject, typically generated by gen_html_data
+    reportdir : str
+        Path to report directory. We will save the plot in this directory.
+
+
+    Returns
+    -------
+    path : str
+        Path to plot.
+    """
+
+    output_file = reportdir / "summary/bad_segs.png"
+    fig, ax = plt.subplots()
+
+    for modality in ['grad', 'mag', 'eeg']:
+
+        if 'bad_seg_pc_' + modality in subject_data[0].keys():
+            pc_segs = []
+
+            for data in subject_data:
+                pc = data['bad_seg_pc_' + modality]
+                pc_segs.append(pc)
+
+            ax.plot(
+                range(1, len(subject_data) + 1),
+                pc_segs,
+                label=modality,
+            )
+
+    ax.legend()
+    ax.set_xlabel("Subject")
+    ax.set_ylabel("Percentage time marked bad")
+    fig.savefig(output_file, dpi=300)
+    plt.close(fig)
+    return f"summary/bad_segs.png"
+
+
+def plot_summary_bad_chans(subject_data, reportdir):
+    """Plot summary of bad channels over subjects.
+
+    Parameters
+    ----------
+    subject_data : list
+        list of data for each subject, typically generated by gen_html_data
+    reportdir : str
+        Path to report directory. We will save the plot in this directory.
+
+    Returns
+    -------
+    path : str
+        Path to plot.
+    """
+
+    output_file = reportdir / "summary/bad_chans.png"
+    fig, ax = plt.subplots()
+
+    # For summary report:
+    num_bads = []
+    for data in subject_data:
+        num_bads.append(data['bad_chan_num'])
+
+    ax.plot(
+        range(1, len(num_bads) + 1),
+        num_bads,
+    )
+    ax.set_xlabel("Subject")
+    ax.set_ylabel("Number of bad channels")
+    fig.savefig(output_file, dpi=300)
+    plt.close(fig)
+    return f"summary/bad_chans.png"
+
 def plot_artefact_scan(raw, savebase=None):
 
     events = mne.find_events(raw, min_duration=2/raw.info['sfreq'])
@@ -1100,22 +1193,3 @@ def print_scan_summary(raw):
         pc = (mod_dur / full_dur) * 100
         s = 'Modality {0} - {1:02f}/{2} seconds rejected     ({3:02f}%)'
         print(s.format(modality, mod_dur, full_dur, pc))
-
-
-def main(argv=None):
-    if argv is None:
-        argv = sys.argv[1:]
-
-    parser = argparse.ArgumentParser(description='Run a quality control summary on data.')
-    parser.add_argument('files', type=str,
-                        help='plain text file containing full paths to files to be processed')
-    parser.add_argument('outdir', type=str, 
-                        help='Path to output directory to save data in')
-
-    args = parser.parse_args(argv)
-
-    gen_report_from_fif(args.files, args.outdir, args.preproc_config)
-
-
-if __name__ == '__main__':
-    main()
