@@ -1,5 +1,3 @@
-#!/usr/bin/env python
-
 """Calculation of surfaces in RHINO.
 
 """
@@ -24,9 +22,9 @@ from mne.transforms import write_trans, Transform
 import osl.source_recon.rhino.utils as rhino_utils
 from osl.utils.logger import log_or_print
 
+
 def get_surfaces_filenames(subjects_dir, subject):
-    """
-    Generates a dict of files generated and used by rhino.compute_surfaces.
+    """Generates a dict of files generated and used by rhino.compute_surfaces.
 
     Files will be in subjects_dir/subject/rhino/surfaces.
 
@@ -41,11 +39,10 @@ def get_surfaces_filenames(subjects_dir, subject):
     -------
     filenames : dict
         A dict of files generated and used by rhino.compute_surfaces.
-
         Note that  due to the unusal naming conventions used by BET:
-         - bet_inskull_*_file is actually the brain surface
-         - bet_outskull_*_file is actually the inner skull surface
-         - bet_outskin_*_file is the outer skin/scalp surface
+        - bet_inskull_*_file is actually the brain surface
+        - bet_outskull_*_file is actually the inner skull surface
+        - bet_outskin_*_file is the outer skin/scalp surface
     """
     basedir = op.join(subjects_dir, subject, "rhino", "surfaces")
     os.makedirs(basedir, exist_ok=True)
@@ -55,18 +52,17 @@ def get_surfaces_filenames(subjects_dir, subject):
         "smri_file": op.join(basedir, "smri.nii.gz"),
         "mni2mri_flirt_xform_file": op.join(basedir, "mni2mri_flirt_xform_file.txt"),
         "mni_mri_t_file": op.join(basedir, "mni_mri-trans.fif"),
-
-        "bet_outskin_mesh_vtk_file": op.join(basedir, "outskin_mesh.vtk"), # BET output
-        "bet_inskull_mesh_vtk_file": op.join(basedir, "inskull_mesh.vtk"), # BET output
-        "bet_outskull_mesh_vtk_file": op.join(basedir, "outskull_mesh.vtk"), # BET output
-
+        "bet_outskin_mesh_vtk_file": op.join(basedir, "outskin_mesh.vtk"),  # BET output
+        "bet_inskull_mesh_vtk_file": op.join(basedir, "inskull_mesh.vtk"),  # BET output
+        "bet_outskull_mesh_vtk_file": op.join(
+            basedir, "outskull_mesh.vtk"
+        ),  # BET output
         "bet_outskin_mesh_file": op.join(basedir, "outskin_mesh.nii.gz"),
         "bet_outskin_plus_nose_mesh_file": op.join(
             basedir, "outskin_plus_nose_mesh.nii.gz"
         ),
         "bet_inskull_mesh_file": op.join(basedir, "inskull_mesh.nii.gz"),
         "bet_outskull_mesh_file": op.join(basedir, "outskull_mesh.nii.gz"),
-
         "std_brain": op.join(
             os.environ["FSLDIR"],
             "data",
@@ -91,8 +87,7 @@ def compute_surfaces(
     subject,
     include_nose=True,
     cleanup_files=True,
-    overwrite=False,
-    logger=None,
+    recompute_surfaces=False,
 ):
     """Compute surfaces.
 
@@ -109,14 +104,14 @@ def compute_surfaces(
     3) Use flirt to register skull stripped sMRI to MNI space
     4) Use BET/BETSURF to get:
     a) The scalp surface (excluding nose), this gives the sMRI-derived
-        headshape points in native sMRI space, which can be used in the
-        headshape points registration later.
+       headshape points in native sMRI space, which can be used in the
+       headshape points registration later.
     b) The scalp surface (outer skin), inner skull and brain surface,
        these can be used for forward modelling later.
        Note that  due to the unusal naming conventions used by BET:
-        - bet_inskull_mesh_file is actually the brain surface
-        - bet_outskull_mesh_file is actually the inner skull surface
-        - bet_outskin_mesh_file is the outer skin/scalp surface
+       - bet_inskull_mesh_file is actually the brain surface
+       - bet_outskull_mesh_file is actually the inner skull surface
+       - bet_outskin_mesh_file is the outer skin/scalp surface
     5) Refine scalp outline, adding nose to scalp surface (optional)
     6) Output the transform from sMRI space to MNI
     7) Output surfaces in sMRI space
@@ -132,11 +127,11 @@ def compute_surfaces(
         will be the native/sMRI axis used throughout rhino. The qform
         will be ignored.
     subjects_dir : string
-        Directory to put RHINO subject dirs in.
-        Files will be in subjects_dir/subject/surfaces/
+        Directory to put RHINO subject directories in.
+        Files will be in subjects_dir/subject/surfaces.
     subject : string
-        Subject name dir to put RHINO files in.
-        Files will be in subjects_dir/subject/surfaces/
+        Subject name directory to put RHINO files in.
+        Files will be in subjects_dir/subject/surfaces.
     include_nose : bool
         Specifies whether to add the nose to the outer skin
         (scalp) surface. This can help rhino's coreg to work
@@ -144,12 +139,11 @@ def compute_surfaces(
         include the nose.
         Requires the smri_file to have a FOV that includes the nose!
     cleanup_files : bool
-        Specifies whether to cleanup intermediate files in the coreg dir.
-    overwrite: bool
-        Specifies whether or not to run compute_surfaces, if the passed in
-        options have already been run
-    logger : logging.getLogger
-        Logger.
+        Specifies whether to cleanup intermediate files in the coreg
+        directory.
+    recompute_surfaces : bool
+        Specifies whether or not to run compute_surfaces if the passed in
+        options have already been run.
     """
 
     # Note the jargon used varies for xforms and coord spaces, e.g.:
@@ -162,7 +156,7 @@ def compute_surfaces(
 
     filenames = get_surfaces_filenames(subjects_dir, subject)
 
-    if not overwrite:
+    if not recompute_surfaces:
         # Check if surfaces have already been computed
         if Path(filenames["completed"]).exists():
             with open(filenames["completed"], "r") as file:
@@ -172,29 +166,17 @@ def compute_surfaces(
                 is_same_mri = completed_mri_file == filenames["smri_file"]
                 is_same_include_nose = completed_include_nose == include_nose
                 if is_same_mri and is_same_include_nose:
-                    log_or_print(
-                        "*** OSL RHINO: USING PREVIOUSLY COMPUTED SURFACES ***",
-                        logger,
-                    )
-                    log_or_print(f"Surfaces directory: {filenames['basedir']}", logger)
-                    log_or_print(f"include_nose={completed_include_nose}", logger)
+                    log_or_print("*** OSL RHINO: USING PREVIOUSLY COMPUTED SURFACES ***")
+                    log_or_print(f"Surfaces directory: {filenames['basedir']}")
+                    log_or_print(f"include_nose={completed_include_nose}")
                     return
 
-    log_or_print("*** RUNNING OSL RHINO COMPUTE SURFACES ***", logger)
+    log_or_print("*** RUNNING OSL RHINO COMPUTE SURFACES ***")
     if include_nose:
-        log_or_print(
-            "The nose is going to be added to the outer skin (scalp) surface.",
-            logger,
-        )
-        log_or_print(
-            "Please ensure that the structural MRI has a FOV that includes the nose",
-            logger,
-        )
+        log_or_print("The nose is going to be added to the outer skin (scalp) surface.")
+        log_or_print("Please ensure that the structural MRI has a FOV that includes the nose")
     else:
-        log_or_print(
-            "The nose is not going to be added to the outer skin (scalp) surface",
-            logger,
-        )
+        log_or_print("The nose is not going to be added to the outer skin (scalp) surface")
 
     # Check smri_file
     smri_ext = "".join(Path(smri_file).suffixes)
@@ -218,7 +200,9 @@ def compute_surfaces(
     if std_orient != "RADIOLOGICAL":
         raise ValueError(
             "Orientation of standard brain must be RADIOLOGICAL, \
-please check output of:\n fslorient -orient {}".format(filenames["std_brain"])
+please check output of:\n fslorient -orient {}".format(
+                filenames["std_brain"]
+            )
         )
 
     # We will assume orientation of sMRI brain is RADIOLOGICAL
@@ -228,31 +212,24 @@ please check output of:\n fslorient -orient {}".format(filenames["std_brain"])
     if smri_orient != "RADIOLOGICAL" and smri_orient != "NEUROLOGICAL":
         raise ValueError(
             "Cannot determine orientation of subject brain, \
-please check output of:\n fslorient -orient {}".format(filenames["smri_file"])
+please check output of:\n fslorient -orient {}".format(
+                filenames["smri_file"]
+            )
         )
 
     # if orientation is not RADIOLOGICAL then force it to be RADIOLOGICAL
     if smri_orient != "RADIOLOGICAL":
-        log_or_print("reorienting subject brain to be RADIOLOGICAL", logger)
+        log_or_print("reorienting subject brain to be RADIOLOGICAL")
         rhino_utils.system_call(
             "fslorient -forceradiological {}".format(filenames["smri_file"])
         )
 
-    log_or_print(
-        "You can use the following call to check the passed in structural MRI is appropriate,",
-        logger,
-    )
-    log_or_print("including checking that the L-R, S-I, A-P labels are sensible:", logger)
-    log_or_print("In Python:", logger)
-    log_or_print(
-        "fsleyes(\"{}\", \"{}\")".format(filenames["smri_file"], filenames["std_brain"]),
-        logger,
-    )
-    log_or_print("From the cmd line:", logger)
-    log_or_print(
-        "fsleyes {} {}".format(filenames["smri_file"], filenames["std_brain"]),
-        logger,
-    )
+    log_or_print("You can use the following call to check the passed in structural MRI is appropriate,")
+    log_or_print("including checking that the L-R, S-I, A-P labels are sensible:")
+    log_or_print("In Python:")
+    log_or_print('fsleyes("{}", "{}")'.format(filenames["smri_file"], filenames["std_brain"]))
+    log_or_print("From the cmd line:")
+    log_or_print("fsleyes {} {}".format(filenames["smri_file"], filenames["std_brain"]))
 
     # -------------------------------------------------------------------------
     # 1) Transform sMRI to be aligned with the MNI axes so that BET works well
@@ -276,9 +253,7 @@ please check output of:\n fslorient -orient {}".format(filenames["smri_file"])
 
     # Apply mri2mniaxes xform to smri to get smri_mniaxes, which means sMRIs
     # voxel indices axes are aligned to be the same as MNI's
-    flirt_smri_mniaxes_file = op.join(
-        filenames["basedir"], "flirt_smri_mniaxes.nii.gz"
-    )
+    flirt_smri_mniaxes_file = op.join(filenames["basedir"], "flirt_smri_mniaxes.nii.gz")
     rhino_utils.system_call(
         "flirt -in {} -ref {} -applyxfm -init {} -out {}".format(
             filenames["smri_file"],
@@ -305,7 +280,7 @@ please check output of:\n fslorient -orient {}".format(filenames["smri_file"])
 
     # -------------------------------------------------------------------------
     # 2) Use BET to skull strip sMRI so that flirt works well
-    log_or_print("Running BET pre-FLIRT...", logger)
+    log_or_print("Running BET pre-FLIRT...")
 
     flirt_smri_mniaxes_bet_file = op.join(
         filenames["basedir"], "flirt_smri_mniaxes_bet"
@@ -316,15 +291,13 @@ please check output of:\n fslorient -orient {}".format(filenames["smri_file"])
 
     # -------------------------------------------------------------------------
     # 3) Use flirt to register skull stripped sMRI to MNI space
-    log_or_print("Running FLIRT...", logger)
+    log_or_print("Running FLIRT...")
 
     # Flirt is run on the skull stripped brains to register the smri_mniaxes
     # to the MNI standard brain
 
     flirt_mniaxes2mni_file = op.join(filenames["basedir"], "flirt_mniaxes2mni.txt")
-    flirt_smri_mni_bet_file = op.join(
-        filenames["basedir"], "flirt_smri_mni_bet.nii.gz"
-    )
+    flirt_smri_mni_bet_file = op.join(filenames["basedir"], "flirt_smri_mni_bet.nii.gz")
     rhino_utils.system_call(
         "flirt -in {} -ref {} -omat {} -o {}".format(
             flirt_smri_mniaxes_bet_file,
@@ -347,7 +320,7 @@ please check output of:\n fslorient -orient {}".format(filenames["smri_file"])
     )
 
     # and also calculate its inverse, from MNI to mri
-    mni2mri_flirt_xform_file = filenames['mni2mri_flirt_xform_file']
+    mni2mri_flirt_xform_file = filenames["mni2mri_flirt_xform_file"]
 
     rhino_utils.system_call(
         "convert_xfm -omat {}  -inverse {}".format(
@@ -380,7 +353,7 @@ please check output of:\n fslorient -orient {}".format(filenames["smri_file"])
 
     # Run BET on smri to get the surface mesh (in MNI space),
     # as BETSURF needs this.
-    log_or_print("Running BET pre-BETSURF...", logger)
+    log_or_print("Running BET pre-BETSURF...")
 
     flirt_smri_mni_bet_file = op.join(filenames["basedir"], "flirt_smri_mni_bet")
     rhino_utils.system_call(
@@ -388,7 +361,7 @@ please check output of:\n fslorient -orient {}".format(filenames["smri_file"])
     )
 
     # Run BETSURF - to get the head surfaces in MNI space
-    log_or_print("Running BETSURF...", logger)
+    log_or_print("Running BETSURF...")
 
     # Need to provide BETSURF with transform to MNI space.
     # Since flirt_smri_mni_file is already in MNI space,
@@ -411,7 +384,7 @@ please check output of:\n fslorient -orient {}".format(filenames["smri_file"])
 
     # -------------------------------------------------------------------------
     # 5) Refine scalp outline, adding nose to scalp surface (optional)
-    log_or_print("Refining scalp surface...", logger)
+    log_or_print("Refining scalp surface...")
 
     # We do this in MNI big FOV space, to allow the full nose to be included
 
@@ -438,9 +411,7 @@ please check output of:\n fslorient -orient {}".format(filenames["smri_file"])
     )
 
     # move MRI to MNI big FOV space and load in
-    flirt_smri_mni_bigfov_file = op.join(
-        filenames["basedir"], "flirt_smri_mni_bigfov"
-    )
+    flirt_smri_mni_bigfov_file = op.join(filenames["basedir"], "flirt_smri_mni_bigfov")
     rhino_utils.system_call(
         "flirt -in {} -ref {} -applyxfm -init {} -out {}".format(
             filenames["smri_file"],
@@ -486,7 +457,7 @@ please check output of:\n fslorient -orient {}".format(filenames["smri_file"])
     mask = mask[1:-1, 1:-1, 1:-1]
 
     if include_nose:
-        log_or_print("Adding nose to scalp surface...", logger)
+        log_or_print("Adding nose to scalp surface...")
 
         # RECLASSIFY BRIGHT VOXELS OUTSIDE OF MASK (TO PUT NOSE INSIDE
         # THE MASK SINCE BET WILL HAVE EXCLUDED IT)
@@ -605,11 +576,12 @@ please check output of:\n fslorient -orient {}".format(filenames["smri_file"])
     # 7) Output surfaces in sMRI(native) space
 
     # Transform betsurf output mask/mesh output from MNI to sMRI space
-    rhino_utils._transform_bet_surfaces(mni2mri_flirt_xform_file,
-                                        filenames["mni_mri_t_file"],
-                                        filenames,
-                                        filenames["smri_file"],
-                                        )
+    rhino_utils._transform_bet_surfaces(
+        mni2mri_flirt_xform_file,
+        filenames["mni_mri_t_file"],
+        filenames,
+        filenames["smri_file"],
+    )
 
     # -------------------------------------------------------------------------
     # Write a file to indicate RHINO has been run
@@ -628,32 +600,38 @@ please check output of:\n fslorient -orient {}".format(filenames["smri_file"])
             "rm -f {}".format(op.join(filenames["basedir"], "flirt*"))
         )
 
-    log_or_print("rhino.surfaces_display(\"{}\", \"{}\") can be used to check the result".format(
-        subjects_dir, subject), logger)
-    log_or_print("*** OSL RHINO COMPUTE SURFACES COMPLETE ***", logger)
+    log_or_print(
+        'rhino.surfaces_display("{}", "{}") can be used to check the result'.format(
+            subjects_dir, subject
+        )
+    )
+    log_or_print("*** OSL RHINO COMPUTE SURFACES COMPLETE ***")
 
 
 def surfaces_display(subjects_dir, subject):
     """Display surfaces.
 
-    Displays the surfaces extracted from the sMRI using rhino.compute_surfaces
-    Display is shown in sMRI (native) space
+    Displays the surfaces extracted from the sMRI using
+    rhino.compute_surfaces.
+
+    Display is shown in sMRI (native) space.
 
     Parameters
     ----------
     subjects_dir : string
-        Directory to put RHINO subject dirs in.
-        Files will be in subjects_dir/subject/surfaces/
+        Directory to put RHINO subject directories in.
+        Files will be in subjects_dir/subject/surfaces.
     subject : string
-        Subject name dir to put RHINO files in.
-        Files will be in subjects_dir/subject/surfaces/
+        Subject name directory to put RHINO files in.
+        Files will be in subjects_dir/subject/surfaces.
 
-    Note that bet_inskull_mesh_file is actually the brain surface and
-    bet_outskull_mesh_file is the inner skull surface, due to the naming
-    conventions used by BET
+    Notes
+    -----
+    bet_inskull_mesh_file is actually the brain surface and
+    bet_outskull_mesh_file is the inner skull surface, due to
+    the naming conventions used by BET.
     """
 
-
     filenames = get_surfaces_filenames(subjects_dir, subject)
 
     rhino_utils.system_call(
