@@ -1,5 +1,3 @@
-#!/usr/bin/env python
-
 """RHINO utilities.
 
 """
@@ -9,7 +7,7 @@
 
 import os
 import os.path as op
-
+import subprocess
 from pathlib import Path
 
 import numpy as np
@@ -35,10 +33,11 @@ from osl.source_recon.beamforming import transform_recon_timeseries
 from osl.utils.logger import log_or_print
 from osl.utils import soft_import
 
+
 def system_call(cmd, verbose=False):
     if verbose:
-        print(cmd)
-    os.system(cmd)
+        log_or_print(cmd)
+    subprocess.call(cmd, shell=True)
 
 
 def get_gridstep(coords):
@@ -52,7 +51,7 @@ def get_gridstep(coords):
     Returns
     -------
     gridstep: int
-        Spatial resolution of dipole grid in mm
+        Spatial resolution of dipole grid in mm.
     """
     store = []
     for ii in range(coords.shape[0]):
@@ -64,20 +63,19 @@ def get_gridstep(coords):
 
 def niimask2indexpointcloud(nii_fname, volindex=None):
     """Takes in a nii.gz mask file name (which equals zero for background and
-    neq zero for the mask) and returns the mask as a 3 x npoints point cloud.
+    != zero for the mask) and returns the mask as a 3 x npoints point cloud.
 
     Parameters
     ----------
     nii_fname : string
-        A nii.gz mask file name
-        (with zero for background, and !=0 for the mask)
+        A nii.gz mask file name (with zero for background, and !=0 for the mask).
     volindex : int
-        Volume index, used if nii_mask is a 4D file
+        Volume index, used if nii_mask is a 4D file.
 
     Returns
     -------
-    pc : nd.array
-        3 x npoints point cloud as voxel indices
+    pc : numpy.ndarray
+        3 x npoints point cloud as voxel indices.
     """
 
     vol = nib.load(nii_fname).get_fdata()
@@ -105,14 +103,14 @@ def niimask2mmpointcloud(nii_mask, volindex=None):
     ----------
     nii_mask : string
         A nii.gz mask file name or the [x,y,z] volume
-        (with zero for background, and !=0 for the mask)
+        (with zero for background, and !=0 for the mask).
     volindex : int
-        Volume index, used if nii_mask is a 4D file
+        Volume index, used if nii_mask is a 4D file.
 
     Returns
     -------
     pc : numpy.ndarray
-        3 x npoints point cloud as mm in native space (using sform)
+        3 x npoints point cloud as mm in native space (using sform).
     values : numpy.ndarray
          npoints values.
     """
@@ -139,9 +137,14 @@ def niimask2mmpointcloud(nii_mask, volindex=None):
 
 
 def _closest_node(node, nodes):
-    """
-    Find nearest node in nodes to the passed in node.
-    Returns the index to the nearest node in nodes.
+    """Find nearest node in nodes to the passed in node.
+
+    Returns
+    -------
+    index : int
+        Index to the nearest node in nodes.
+    distance : float
+        Distance.
     """
 
     if len(nodes) == 1:
@@ -153,6 +156,30 @@ def _closest_node(node, nodes):
     return index, distance
 
 
+def _get_vol_info_from_nii(mri):
+    """Read volume info from an MRI file.
+
+    Parameters
+    ----------
+    mri : str
+        Path to MRI file.
+
+    Returns
+    -------
+    out : dict
+        Dictionary with keys 'mri_width', 'mri_height', 'mri_depth'
+        and 'mri_volume_name'.
+    """
+    dims = nib.load(mri).get_fdata().shape
+    out = dict(
+        mri_width=dims[0],
+        mri_height=dims[1],
+        mri_depth=dims[2],
+        mri_volume_name=mri,
+    )
+    return out
+
+
 def _get_sform(nii_file):
     """
     sform allows mapping from simple voxel index cordinates
@@ -254,15 +281,15 @@ def _binary_majority3d(img):
     return imgout
 
 
-def rigid_transform_3D(B, A, compute_scaling = False):
+def rigid_transform_3D(B, A, compute_scaling=False):
     """Calculate affine transform from points in A to point in B.
 
     Parameters
     ----------
     A : numpy.ndarray
-        3 x num_points. Set of points to register from
+        3 x num_points. Set of points to register from.
     B : numpy.ndarray
-        3 x num_points. Set of points to register to
+        3 x num_points. Set of points to register to.
 
     compute_scaling : bool
         Do we compute a scaling on top of rotation and translation?
@@ -270,9 +297,10 @@ def rigid_transform_3D(B, A, compute_scaling = False):
     Returns
     -------
     xform : numpy.ndarray
-        Calculated affine transform, does not include scaling
+        Calculated affine transform, does not include scaling.
     scaling_xform : numpy.ndarray
-        Calculated scaling transform (a diagonal 4x4 matrix), does not include rotation or translation
+        Calculated scaling transform (a diagonal 4x4 matrix),
+        does not include rotation or translation.
 
     see http://nghiaho.com/?page_id=671
     """
@@ -305,7 +333,6 @@ def rigid_transform_3D(B, A, compute_scaling = False):
     # if linalg.matrix_rank(H) < 3:
     #    raise ValueError("rank of H = {}, expecting 3".format(linalg.matrix_rank(H)))
 
-
     # find rotation
     U, S, Vt = np.linalg.svd(H)
     R = Vt.T @ U.T
@@ -322,14 +349,14 @@ def rigid_transform_3D(B, A, compute_scaling = False):
         # Bm = C @ R @ Am + error
         # where C = c I is a scalar scaling
         RAm = R @ Am
-        U2, S2, V2t = np.linalg.svd( Bm @ np.linalg.pinv(RAm) )
+        U2, S2, V2t = np.linalg.svd(Bm @ np.linalg.pinv(RAm))
 
         # take average of eigenvalues, accounting for the rank
-        S2 = np.identity(3)*np.mean(S2[S2 > 1e-9])
+        S2 = np.identity(3) * np.mean(S2[S2 > 1e-9])
 
         scaling_xform[0:3, 0:3] = S2
 
-    t = - R @ centroid_A + centroid_B
+    t = -R @ centroid_A + centroid_B
 
     xform = np.eye(4)
     xform[0:3, 0:3] = R
@@ -337,20 +364,21 @@ def rigid_transform_3D(B, A, compute_scaling = False):
 
     return xform, scaling_xform
 
+
 def xform_points(xform, pnts):
     """Applies homogenous linear transformation to an array of 3D coordinates.
 
     Parameters
     ----------
     xform : numpy.ndarray
-        4x4 matrix containing the affine transform
+        4x4 matrix containing the affine transform.
     pnts : numpy.ndarray
-        points to transform, should be 3 x num_points
+        points to transform, should be 3 x num_points.
 
     Returns
     -------
     newpnts : numpy.ndarray
-        pnts following the xform, will be 3 x num_points
+        pnts following the xform, will be 3 x num_points.
     """
     if len(pnts.shape) == 1:
         pnts = np.reshape(pnts, [-1, 1])
@@ -422,16 +450,16 @@ def nearest_neighbor(src, dst):
     Parameters
     ----------
     src : numpy.ndarray
-        Nxm array of points
+        Nxm array of points.
     dst : numpy.ndarray
-        Nxm array of points
+        Nxm array of points.
 
     Returns
     -------
     distances : numpy.ndarray
-        Euclidean distances of the nearest neighbor
+        Euclidean distances of the nearest neighbor.
     indices : numpy.ndarray
-        dst indices of the nearest neighbor
+        dst indices of the nearest neighbor.
     """
 
     neigh = NearestNeighbors(n_neighbors=1)
@@ -447,24 +475,24 @@ def icp(A, B, init_pose=None, max_iterations=50, tolerance=0.0001):
     Parameters
     ----------
     A : numpy.ndarray
-        Nxm numpy array of source mD points
+        Nxm numpy array of source mD points.
     B : numpy.ndarray
-        Nxm numpy array of destination mD point
+        Nxm numpy array of destination mD point.
     init_pose : numpy.ndarray
-        (m+1)x(m+1) homogeneous transformation
+        (m+1)x(m+1) homogeneous transformation.
     max_iterations : int
-        Exit algorithm after max_iterations
+        Exit algorithm after max_iterations.
     tolerance : float
-        Convergence criteria
+        Convergence criteria.
 
     Returns
     -------
     T : numpy.ndarray
-        (4 x 4) Final homogeneous transformation that maps A on to B
+        (4 x 4) Final homogeneous transformation that maps A on to B.
     distances : numpy.ndarray
-        Euclidean distances (errors) of the nearest neighbor
+        Euclidean distances (errors) of the nearest neighbor.
     i : float
-        Number of iterations to converge
+        Number of iterations to converge.
 
     Notes
     -----
@@ -514,10 +542,8 @@ def icp(A, B, init_pose=None, max_iterations=50, tolerance=0.0001):
     return T, distances, i
 
 
-def rhino_icp(
-    smri_headshape_polhemus, polhemus_headshape_polhemus, Ninits=10, logger=None
-):
-    """Runs Iterative Closest Point with multiple initialisations.
+def rhino_icp(smri_headshape_polhemus, polhemus_headshape_polhemus, n_init=10):
+    """Runs Iterative Closest Point (ICP) with multiple initialisations.
 
     Parameters
     ----------
@@ -525,11 +551,9 @@ def rhino_icp(
         [3 x N] locations of the Headshape points in polehumus space
         (i.e. MRI scalp surface).
     polhemus_headshape_polhemus : numpy.ndarray
-        [3 x N] locations of the Polhemus headshape points in polhemus space
-    Ninits : int
+        [3 x N] locations of the Polhemus headshape points in polhemus space.
+    n_init : int
         Number of random initialisations to perform.
-    logger : logging.getLogger
-        Logger.
 
     Returns
     -------
@@ -548,7 +572,7 @@ def rhino_icp(
     data2 = polhemus_headshape_polhemus
 
     err_old = np.Infinity
-    err = np.zeros(Ninits)
+    err = np.zeros(n_init)
 
     Mr = np.eye(4)
 
@@ -558,7 +582,7 @@ def rhino_icp(
 
     data2r = data2
 
-    for init in range(Ninits):
+    for init in range(n_init):
 
         Mi, distances, i = icp(data2r.T, data1.T)
 
@@ -568,7 +592,7 @@ def rhino_icp(
 
         if err[init] < err_old:
 
-            log_or_print("ICP found better xform, error={}".format(e), logger)
+            log_or_print("ICP found better xform, error={}".format(e))
 
             err_old = e
 
@@ -586,8 +610,12 @@ def rhino_icp(
                 plt.figure(frameon=False)
                 ax = plt.axes(projection="3d")
                 ax.scatter(
-                    data1[0, 0:-1:10], data1[1, 0:-1:10], data1[2, 0:-1:10], c="blue",
-                    marker=".", s=1,
+                    data1[0, 0:-1:10],
+                    data1[1, 0:-1:10],
+                    data1[2, 0:-1:10],
+                    c="blue",
+                    marker=".",
+                    s=1,
                 )
                 ax.scatter(
                     data2[0, :], data2[1, :], data2[2, :], c="red", marker="o", s=5
@@ -596,8 +624,12 @@ def rhino_icp(
                     data2r[0, :], data2r[1, :], data2r[2, :], c="green", marker="o", s=5
                 )
                 ax.scatter(
-                    data2r_icp[0, :], data2r_icp[1, :], data2r_icp[2, :], c="yellow",
-                    marker="o", s=5,
+                    data2r_icp[0, :],
+                    data2r_icp[1, :],
+                    data2r_icp[2, :],
+                    c="yellow",
+                    marker="o",
+                    s=5,
                 )
                 plt.show()
                 plt.draw()
@@ -652,7 +684,7 @@ def _get_vtk_mesh_native(vtk_mesh_file, nii_mesh_file):
     num_rrs = int(data.iloc[3, 1])
 
     # these will be in voxel index space
-    rrs_flirtcoords = data.iloc[4: num_rrs + 4, 0:3].to_numpy().astype(np.float64)
+    rrs_flirtcoords = data.iloc[4 : num_rrs + 4, 0:3].to_numpy().astype(np.float64)
 
     # move from flirtcoords mm to mri mm (native) space
     xform_flirtcoords2nii = _get_flirtcoords2native_xform(nii_mesh_file)
@@ -660,7 +692,7 @@ def _get_vtk_mesh_native(vtk_mesh_file, nii_mesh_file):
 
     num_tris = int(data.iloc[num_rrs + 4, 1])
     tris_nii = (
-        data.iloc[num_rrs + 5: num_rrs + 5 + num_tris, 1:4].to_numpy().astype(int)
+        data.iloc[num_rrs + 5 : num_rrs + 5 + num_tris, 1:4].to_numpy().astype(int)
     )
 
     return rrs_nii, tris_nii
@@ -743,7 +775,7 @@ def _transform_vtk_mesh(
     data = pd.read_csv(vtk_mesh_file_in, delim_whitespace=True)
 
     num_rrs = int(data.iloc[3, 1])
-    data.iloc[4: num_rrs + 4, 0:3] = rrs_out
+    data.iloc[4 : num_rrs + 4, 0:3] = rrs_out
 
     # write new vtk file
     data.to_csv(out_vtk_file, sep=" ", index=False)
@@ -751,10 +783,11 @@ def _transform_vtk_mesh(
 
 def _get_mne_xform_from_flirt_xform(flirt_xform, nii_mesh_file_in, nii_mesh_file_out):
     """
-    Returns a mm coordinates to mm coordinates MNE xform that corresponds to the passed in flirt xform
+    Returns a mm coordinates to mm coordinates MNE xform that corresponds to the
+    passed in flirt xform.
 
     Note that we need to do this as flirt xforms include an extra xform
-    based on the voxel dimensions (see _get_flirtcoords2native_xform )
+    based on the voxel dimensions (see _get_flirtcoords2native_xform).
     """
 
     flirtcoords2native_xform_in = _get_flirtcoords2native_xform(nii_mesh_file_in)
@@ -784,9 +817,8 @@ def _get_flirt_xform_between_axes(from_nii, target_nii):
       from2targetaxes = inv(targetvox2target) * fromvox2from
 
     In more detail:
-    We need the sform for the transformed from_nii
-    to be the same as the sform for the target_nii, without changing the
-    actual coordinates (in mm).
+    We need the sform for the transformed from_nii to be the same as the sform
+    for the target_nii, without changing the actual coordinates (in mm).
     In other words, we need:
     fromvox2from * from_nii_vox = targetvox2target * from_nii_target_vox
     where
@@ -898,14 +930,13 @@ def recon_timeseries2niftii(
     Parameters
     ----------
     subjects_dir : string
-        Directory to find RHINO subject dirs in.
+        Directory to find RHINO subject directories in.
     subject : string
-        Subject name dir to find RHINO files in.
-    recon_timeseries : (ndipoles, ntpts) np.array
+        Subject name directory to find RHINO files in.
+    recon_timeseries : (ndipoles, ntpts) np.ndarray
         Reconstructed time courses (in head (polhemus) space).
         Assumes that the dipoles are the same (and in the same order)
-        as those in the forward model,
-        coreg_filenames['forward_model_file'].
+        as those in the forward model, coreg_filenames['forward_model_file'].
         Typically derive from the VolSourceEstimate's output by
         MNE source recon methods, e.g. mne.beamformer.apply_lcmv, obtained
         using a forward model generated by RHINO.
@@ -915,20 +946,19 @@ def recon_timeseries2niftii(
         If None, then the gridstep used in coreg_filenames['forward_model_file']
         is used.
     reference_brain : string, 'mni' or 'mri'
-        'mni' indicates that the reference_brain is the stdbrain in MNI space
-        'mri' indicates that the reference_brain is the sMRI in native/mri space
-    times : (ntpts, ) np.array
-        Times points in seconds.
-        Will assume that these are regularly spaced
+        'mni' indicates that the reference_brain is the stdbrain in MNI space.
+        'mri' indicates that the reference_brain is the sMRI in native/mri space.
+    times : (ntpts, ) np.ndarray
+        Times points in seconds. Will assume that these are regularly spaced.
 
     Returns
     -------
     out_nii_fname : string
-        Name of output niftii file
+        Name of output niftii file.
     reference_brain_fname : string
-        Niftii file name of standard brain mask in MNI space at requested resolution,
-        int(stdbrain_resolution)
-        (with zero for background, and !=0 for the mask)
+        Niftii file name of standard brain mask in MNI space at requested
+        resolution, int(stdbrain_resolution) (with zero for background, and
+        !=0 for the mask).
     """
 
     if len(recon_timeseries.shape) == 1:
@@ -963,7 +993,7 @@ def recon_timeseries2niftii(
     return out_nii_fname, reference_brain_fname
 
 
-def save_or_show_renderer(renderer, filename, logger=None):
+def save_or_show_renderer(renderer, filename):
     """Save or show a renderer.
 
     Parameters
@@ -973,8 +1003,6 @@ def save_or_show_renderer(renderer, filename, logger=None):
     filename : str
         Filename to save display to (as an interactive html).
         Must have extension .html. If None we display the renderer.
-    logger : logging.getLogger
-        Logger
     """
     if filename is None:
         renderer.show()
@@ -987,7 +1015,7 @@ def save_or_show_renderer(renderer, filename, logger=None):
                 + " ".join(allowed_extensions)
             )
 
-        log_or_print(f"saving {filename}", logger)
+        log_or_print(f"saving {filename}")
         if ext == ".html":
             renderer.figure.plotter.export_html(filename)
         elif ext in allowed_extensions:
@@ -999,20 +1027,26 @@ def _create_freesurfer_meshes_from_bet_surfaces(filenames, xform_mri_voxel2mri):
     # Create sMRI-derived freesurfer surfaces in native/mri space in mm,
     # for use by forward modelling
 
-    _create_freesurfer_mesh_from_bet_surface(infile=filenames['bet_inskull_mesh_vtk_file'],
-                                       surf_outfile=filenames['bet_inskull_surf_file'],
-                                       nii_mesh_file=filenames['bet_inskull_mesh_file'],
-                                       xform_mri_voxel2mri=xform_mri_voxel2mri)
+    _create_freesurfer_mesh_from_bet_surface(
+        infile=filenames["bet_inskull_mesh_vtk_file"],
+        surf_outfile=filenames["bet_inskull_surf_file"],
+        nii_mesh_file=filenames["bet_inskull_mesh_file"],
+        xform_mri_voxel2mri=xform_mri_voxel2mri,
+    )
 
-    _create_freesurfer_mesh_from_bet_surface(infile=filenames['bet_outskull_mesh_vtk_file'],
-                                       surf_outfile=filenames['bet_outskull_surf_file'],
-                                       nii_mesh_file=filenames['bet_outskull_mesh_file'],
-                                       xform_mri_voxel2mri=xform_mri_voxel2mri)
+    _create_freesurfer_mesh_from_bet_surface(
+        infile=filenames["bet_outskull_mesh_vtk_file"],
+        surf_outfile=filenames["bet_outskull_surf_file"],
+        nii_mesh_file=filenames["bet_outskull_mesh_file"],
+        xform_mri_voxel2mri=xform_mri_voxel2mri,
+    )
 
-    _create_freesurfer_mesh_from_bet_surface(infile=filenames['bet_outskin_mesh_vtk_file'],
-                                       surf_outfile=filenames['bet_outskin_surf_file'],
-                                       nii_mesh_file=filenames['bet_outskin_mesh_file'],
-                                       xform_mri_voxel2mri=xform_mri_voxel2mri)
+    _create_freesurfer_mesh_from_bet_surface(
+        infile=filenames["bet_outskin_mesh_vtk_file"],
+        surf_outfile=filenames["bet_outskin_surf_file"],
+        nii_mesh_file=filenames["bet_outskin_mesh_file"],
+        xform_mri_voxel2mri=xform_mri_voxel2mri,
+    )
 
 
 def _create_freesurfer_mesh_from_bet_surface(
@@ -1068,7 +1102,9 @@ def _create_freesurfer_mesh_from_bet_surface(
         # or you might want to flip the normals to make them point outward, not mandatory
         pcd.normals = o3d.utility.Vector3dVector(-np.asarray(pcd.normals))
 
-        mesh = o3d.geometry.TriangleMesh.create_from_point_cloud_poisson(pcd, depth=8)[0]
+        mesh = o3d.geometry.TriangleMesh.create_from_point_cloud_poisson(pcd, depth=8)[
+            0
+        ]
 
         # mesh = mesh.simplify_quadric_decimation(nii_nativeindex.shape[1])
 
@@ -1101,7 +1137,6 @@ def _create_freesurfer_mesh_from_bet_surface(
         raise ValueError("Invalid infile. Needs to be a .nii.gz or .vtk file")
 
 
-
 def _transform_bet_surfaces(flirt_xform_file, mne_xform_file, filenames, smri_file):
 
     # Transform betsurf mask/mesh using passed in flirt transform and mne transform
@@ -1109,10 +1144,10 @@ def _transform_bet_surfaces(flirt_xform_file, mne_xform_file, filenames, smri_fi
         # xform mask
         system_call(
             "flirt -interp nearestneighbour -in {} -ref {} -applyxfm -init {} -out {}".format(
-            op.join(filenames["basedir"], "flirt_" + mesh_name + ".nii.gz"),
-            smri_file,
-            flirt_xform_file,
-            op.join(filenames["basedir"], mesh_name),
+                op.join(filenames["basedir"], "flirt_" + mesh_name + ".nii.gz"),
+                smri_file,
+                flirt_xform_file,
+                op.join(filenames["basedir"], mesh_name),
             )
         )
 
