@@ -6,7 +6,7 @@ section of a config.
 All functions in this module accept the following arguments for
 consistency:
 
-    func(src_dir, subject, preproc_file, smri_file, *userargs, logger)
+    func(src_dir, subject, preproc_file, smri_file, epoch_file, *userargs)
 
 Custom functions (i.e. functions passed via the extra_funcs argument)
 must also conform to this.
@@ -15,17 +15,17 @@ must also conform to this.
 # Authors: Chetan Gohil <chetan.gohil@psych.ox.ac.uk>
 
 
-import logging
 import os.path as op
 from pathlib import Path
 import pickle
 
 import numpy as np
+import mne
 
 from . import rhino, beamforming, parcellation, sign_flipping
 from ..report import src_report
 
-
+import logging
 logger = logging.getLogger(__name__)
 
 
@@ -38,7 +38,7 @@ def extract_fiducials_from_fif(
     subject,
     preproc_file,
     smri_file,
-    logger,
+    epoch_file,
     **userargs,
 ):
     """Wrapper function to extract fiducials/headshape points.
@@ -53,8 +53,8 @@ def extract_fiducials_from_fif(
         Path to the preprocessed fif file.
     smri_file : str
         Path to the T1 weighted structural MRI file to use in source reconstruction.
-    logger : logging.getLogger
-        Logger.
+    epoch_file : str
+        Path to epoched preprocessed fif file.
     userargs : keyword arguments
         Keyword arguments to pass to
         osl.source_recon.rhino.extract_polhemus_from_info.
@@ -81,9 +81,9 @@ def compute_surfaces(
     subject,
     preproc_file,
     smri_file,
-    logger,
+    epoch_file,
     include_nose,
-    overwrite=False,
+    recompute_surfaces=False,
 ):
     """Wrapper for computing surfaces.
 
@@ -97,11 +97,11 @@ def compute_surfaces(
         Path to the preprocessed fif file.
     smri_file : str
         Path to the T1 weighted structural MRI file to use in source reconstruction.
-    logger : logging.getLogger
-        Logger.
+    epoch_file : str
+        Path to epoched preprocessed fif file.
     include_nose : bool
         Should we include the nose when we're extracting the surfaces?
-    overwrite: bool
+    recompute_surfaces : bool
         Specifies whether or not to run compute_surfaces, if the passed in
         options have already been run
     """
@@ -111,8 +111,7 @@ def compute_surfaces(
         subjects_dir=src_dir,
         subject=subject,
         include_nose=include_nose,
-        overwrite=overwrite,
-        logger=logger,
+        recompute_surfaces=recompute_surfaces,
     )
 
     # Save info for the report
@@ -130,11 +129,12 @@ def coreg(
     subject,
     preproc_file,
     smri_file,
-    logger,
+    epoch_file,
     use_nose,
     use_headshape,
     already_coregistered=False,
     allow_smri_scaling=False,
+    n_init=30,
 ):
     """Wrapper for full coregistration: compute_surfaces, coreg and forward_model.
 
@@ -148,8 +148,8 @@ def coreg(
         Path to the preprocessed fif file.
     smri_file : str
         Path to the T1 weighted structural MRI file to use in source reconstruction.
-    logger : logging.getLogger
-        Logger.
+    epoch_file : str
+        Path to epoched preprocessed fif file.
     use_nose : bool
         Should we use the nose in the coregistration?
     use_headshape : bool
@@ -163,6 +163,8 @@ def coreg(
         but not the size of the sMRI-derived fids.
         E.g. this might be the case if we do not trust the size (e.g. in mm) of the sMRI,
         or if we are using a template sMRI that has not come from this subject.
+    n_init : int
+        Number of initialisation for coregistration.
     """
     # Run coregistration
     rhino.coreg(
@@ -173,16 +175,21 @@ def coreg(
         use_nose=use_nose,
         already_coregistered=already_coregistered,
         allow_smri_scaling=allow_smri_scaling,
-        logger=logger,
+        n_init=n_init,
     )
 
+    # Calculate metrics
+    if already_coregistered:
+        fid_err = None
+    else:
+        fid_err = rhino.coreg_metrics(subjects_dir=src_dir, subject=subject)
+
     # Save plots
     rhino.coreg_display(
         subjects_dir=src_dir,
         subject=subject,
         display_outskin_with_nose=False,
         filename=f"{src_dir}/{subject}/rhino/coreg.html",
-        logger=logger,
     )
     
     # Save info for the report
@@ -194,6 +201,8 @@ def coreg(
             "use_nose": use_nose,
             "already_coregistered": already_coregistered,
             "allow_smri_scaling": allow_smri_scaling,
+            "n_init_coreg": n_init,
+            "fid_err": fid_err,
             "coreg_plot": f"{src_dir}/{subject}/rhino/coreg.html",
         }
     )
@@ -204,7 +213,7 @@ def forward_model(
     subject,
     preproc_file,
     smri_file,
-    logger,
+    epoch_file,
     model,
     eeg=False,
 ):
@@ -220,8 +229,8 @@ def forward_model(
         Path to the preprocessed fif file.
     smri_file : str
         Path to the T1 weighted structural MRI file to use in source reconstruction.
-    logger : logging.getLogger
-        Logger.
+    epoch_file : str
+        Path to epoched preprocessed fif file.
     eeg : bool
         Are we using EEG channels in the source reconstruction?
     """
@@ -231,7 +240,6 @@ def forward_model(
         subject=subject,
         model=model,
         eeg=eeg,
-        logger=logger,
     )
 
     # Save info for the report
@@ -250,14 +258,15 @@ def coregister(
     subject,
     preproc_file,
     smri_file,
-    logger,
+    epoch_file,
     include_nose,
     use_nose,
     use_headshape,
     model,
-    overwrite=False,
+    recompute_surfaces=False,
     already_coregistered=False,
     allow_smri_scaling=False,
+    n_init=30,
     eeg=False,
 ):
     """Wrapper for full coregistration: compute_surfaces, coreg and forward_model.
@@ -272,8 +281,8 @@ def coregister(
         Path to the preprocessed fif file.
     smri_file : str
         Path to the T1 weighted structural MRI file to use in source reconstruction.
-    logger : logging.getLogger
-        Logger.
+    epoch_file : str
+        Path to epoched preprocessed fif file.
     include_nose : bool
         Should we include the nose when we're extracting the surfaces?
     use_nose : bool
@@ -282,7 +291,7 @@ def coregister(
         Should we use the headshape points in the coregistration?
     model : str
         Forward model to use.
-    overwrite : bool
+    recompute_surfaces : bool
         Specifies whether or not to run compute_surfaces, if the passed in
         options have already been run
     already_coregistered : bool
@@ -296,6 +305,8 @@ def coregister(
         or if we are using a template sMRI that has not come from this subject.
     eeg : bool
         Are we using EEG channels in the source reconstruction?
+    n_init : int
+        Number of initialisation for coregistration.
     """
     # Compute surfaces
     rhino.compute_surfaces(
@@ -303,8 +314,7 @@ def coregister(
         subjects_dir=src_dir,
         subject=subject,
         include_nose=include_nose,
-        overwrite=overwrite,
-        logger=logger,
+        recompute_surfaces=recompute_surfaces,
     )
 
     # Run coregistration
@@ -316,11 +326,14 @@ def coregister(
         use_nose=use_nose,
         already_coregistered=already_coregistered,
         allow_smri_scaling=allow_smri_scaling,
-        logger=logger,
+        n_init=n_init,
     )
 
     # Calculate metrics
-    fid_err = rhino.coreg_metrics(subjects_dir=src_dir, subject=subject)
+    if already_coregistered:
+        fid_err = None
+    else:
+        fid_err = rhino.coreg_metrics(subjects_dir=src_dir, subject=subject)
 
     # Save plots
     rhino.coreg_display(
@@ -328,7 +341,6 @@ def coregister(
         subject=subject,
         display_outskin_with_nose=False,
         filename=f"{src_dir}/{subject}/rhino/coreg.html",
-        logger=logger,
     )
 
     # Compute forward model
@@ -337,7 +349,6 @@ def coregister(
         subject=subject,
         model=model,
         eeg=eeg,
-        logger=logger,
     )
 
     # Save info for the report
@@ -353,6 +364,7 @@ def coregister(
             "use_headshape": use_headshape,
             "already_coregistered": already_coregistered,
             "allow_smri_scaling": allow_smri_scaling,
+            "n_init_coreg": n_init,
             "forward_model": True,
             "model": model,
             "eeg": eeg,
@@ -371,10 +383,13 @@ def beamform(
     subject,
     preproc_file,
     smri_file,
-    logger,
+    epoch_file,
+    freq_range,
     chantypes,
     rank,
-    freq_range,
+    spatial_resolution=None,
+    reference_brain="mni",
+    save_bf_data=False,
 ):
     """Wrapper function for beamforming.
 
@@ -388,26 +403,48 @@ def beamform(
         Path to the preprocessed fif file.
     smri_file : str
         Path to the T1 weighted structural MRI file to use in source reconstruction.
-    logger : logging.getLogger
-        Logger.
+    epoch_file : str
+        Path to epoched preprocessed fif file.
+    freq_range : list
+        Lower and upper band to bandpass filter before beamforming. If None,
+        no filtering is done.
     chantypes : list of str
         Channel types to use in beamforming.
     rank : dict
         Keys should be the channel types and the value should be the rank to use.
-    freq_range : list
-        Lower and upper band to bandpass filter before beamforming. If None,
-        no filtering is done.
+    spatial_resolution : int
+        Resolution for beamforming to use for the reference brain in mm
+        (must be an integer, or will be cast to nearest int)
+        If None, then the gridstep used in coreg_filenames['forward_model_file']
+        is used.
+    reference_brain : string
+        'mni' indicates that the reference_brain is the stdbrain in MNI space
+        'mri' indicates that the reference_brain is the subject's sMRI in
+            the scaled native/mri space. "
+        'unscaled_mri' indicates that the reference_brain is the subject's sMRI in
+            unscaled native/mri space.
+        Note that Scaled/unscaled relates to the allow_smri_scaling option in coreg.
+        If allow_scaling was False, then the unscaled MRI will be the same as the scaled.
+        MRI.
+    save_bf_data : bool
+        Should we save the beamformed data?
     """
-    from ..preprocessing import import_data
+    logger.info("beamforming")
 
-    # Load preprocessed data
-    preproc_data = import_data(preproc_file)
-    preproc_data.pick(chantypes)
+    if epoch_file is not None:
+        logger.info("using epoched data")
+
+        # Load epoched data
+        data = mne.read_epochs(epoch_file, preload=True)
+    else:
+        # Load preprocessed data
+        data = mne.io.read_raw_fif(preproc_file, preload=True)
+    data.pick(chantypes)
 
     if freq_range is not None:
         # Bandpass filter
         logger.info("bandpass filtering: {}-{} Hz".format(freq_range[0], freq_range[1]))
-        preproc_data = preproc_data.filter(
+        data = data.filter(
             l_freq=freq_range[0],
             h_freq=freq_range[1],
             method="iir",
@@ -425,23 +462,38 @@ def beamform(
     filters = beamforming.make_lcmv(
         subjects_dir=src_dir,
         subject=subject,
-        data=preproc_data,
+        data=data,
         chantypes=chantypes,
         weight_norm="nai",
         rank=rank,
-        logger=logger,
         save_figs=True,
     )
 
+    # Save the beamforming filter
+    filters_file = src_dir / subject / "rhino/filters-lcmv.h5"
+    filters.save(filters_file, overwrite=True)
+
     # Apply beamforming
-    logger.info("beamforming.apply_lcmv_raw")
-    src_data = beamforming.apply_lcmv_raw(preproc_data, filters)
-    src_ts_mni, _, src_coords_mni, _ = beamforming.transform_recon_timeseries(
+    logger.info("beamforming.apply_lcmv")
+    bf_data = beamforming.apply_lcmv(data, filters)
+    if epoch_file is not None:
+        bf_data = np.transpose([bf.data for bf in bf_data], axes=[1, 2, 0])
+    else:
+        bf_data = bf_data.data
+    bf_data_mni, _, _, _ = beamforming.transform_recon_timeseries(
         subjects_dir=src_dir,
         subject=subject,
-        recon_timeseries=src_data.data,
+        recon_timeseries=bf_data,
+        spatial_resolution=spatial_resolution,
+        reference_brain=reference_brain,
     )
 
+    if save_bf_data:
+        # Save beamformed data
+        bf_data_file = src_dir / subject / "rhino/bf.npy"
+        logger.info(f"saving {bf_data_file}")
+        np.save(bf_data_file, bf_data_mni.T)
+
     # Save info for the report
     src_report.add_to_data(
         f"{src_dir}/{subject}/report_data.pkl",
@@ -461,13 +513,16 @@ def beamform_and_parcellate(
     subject,
     preproc_file,
     smri_file,
-    logger,
+    epoch_file,
     chantypes,
     rank,
     freq_range,
     parcellation_file,
     method,
     orthogonalisation,
+    spatial_resolution=None,
+    reference_brain="mni",
+    save_bf_data=False,
 ):
     """Wrapper function for beamforming and parcellation.
 
@@ -481,8 +536,8 @@ def beamform_and_parcellate(
         Path to the preprocessed fif file.
     smri_file : str
         Path to the T1 weighted structural MRI file to use in source reconstruction.
-    logger : logging.getLogger
-        Logger.
+    epoch_file : str
+        Path to epoched preprocessed fif file.
     chantypes : list of str
         Channel types to use in beamforming.
     rank : dict
@@ -496,17 +551,39 @@ def beamform_and_parcellate(
         Method to use in the parcellation.
     orthogonalisation : bool
         Should we do orthogonalisation?
+    spatial_resolution : int
+        Resolution for beamforming to use for the reference brain in mm
+        (must be an integer, or will be cast to nearest int)
+        If None, then the gridstep used in coreg_filenames['forward_model_file']
+        is used.
+    reference_brain : string
+        'mni' indicates that the reference_brain is the stdbrain in MNI space
+        'mri' indicates that the reference_brain is the subject's sMRI in
+            the scaled native/mri space. "
+        'unscaled_mri' indicates that the reference_brain is the subject's sMRI in
+            unscaled native/mri space.
+        Note that Scaled/unscaled relates to the allow_smri_scaling option in coreg.
+        If allow_scaling was False, then the unscaled MRI will be the same as the scaled.
+        MRI.
+    save_bf_data : bool
+        Should we save the beamformed data?
     """
-    from ..preprocessing import import_data
+    logger.info("beamform_and_parcellate")
 
-    # Load preprocessed data
-    preproc_data = import_data(preproc_file)
-    preproc_data.pick(chantypes)
+    if epoch_file is not None:
+        logger.info("using epoched data")
+
+        # Load epoched data
+        data = mne.read_epochs(epoch_file, preload=True)
+    else:
+        # Load preprocessed data
+        data = mne.io.read_raw_fif(preproc_file, preload=True)
+    data.pick(chantypes)
 
     if freq_range is not None:
         # Bandpass filter
         logger.info("bandpass filtering: {}-{} Hz".format(freq_range[0], freq_range[1]))
-        preproc_data = preproc_data.filter(
+        data = data.filter(
             l_freq=freq_range[0],
             h_freq=freq_range[1],
             method="iir",
@@ -524,58 +601,81 @@ def beamform_and_parcellate(
     filters = beamforming.make_lcmv(
         subjects_dir=src_dir,
         subject=subject,
-        data=preproc_data,
+        data=data,
         chantypes=chantypes,
         weight_norm="nai",
         rank=rank,
-        logger=logger,
         save_figs=True,
     )
 
+    # Save the beamforming filter
+    filters_file = src_dir / subject / "rhino/filters-lcmv.h5"
+    filters.save(filters_file, overwrite=True)
+
     # Apply beamforming
-    logger.info("beamforming.apply_lcmv_raw")
-    src_data = beamforming.apply_lcmv_raw(preproc_data, filters)
-    src_ts_mni, _, src_coords_mni, _ = beamforming.transform_recon_timeseries(
+    logger.info("beamforming.apply_lcmv")
+    bf_data = beamforming.apply_lcmv(data, filters)
+    if epoch_file is not None:
+        bf_data = np.transpose([bf.data for bf in bf_data], axes=[1, 2, 0])
+    else:
+        bf_data = bf_data.data
+    bf_data_mni, _, coords_mni, _ = beamforming.transform_recon_timeseries(
         subjects_dir=src_dir,
         subject=subject,
-        recon_timeseries=src_data.data,
+        recon_timeseries=bf_data,
+        spatial_resolution=spatial_resolution,
+        reference_brain=reference_brain,
     )
 
+    if save_bf_data:
+        # Save beamformed data
+        bf_data_file = src_dir / subject / "rhino/bf.npy"
+        logger.info(f"saving {bf_data_file}")
+        np.save(bf_data_file, bf_data_mni.T)
+
     # Parcellation
     logger.info("parcellation")
-    logger.info(parcellation_file)
-    p = parcellation.Parcellation(parcellation_file)
-    p.parcellate(
-        voxel_timeseries=src_ts_mni,
-        voxel_coords=src_coords_mni,
+    logger.info(f"using file {parcellation_file}")
+    parcel_data, _, _ = parcellation.parcellate_timeseries(
+        parcellation_file,
+        voxel_timeseries=bf_data_mni,
+        voxel_coords=coords_mni,
         method=method,
-        logger=logger,
+        working_dir=src_dir / subject / "rhino",
     )
-    parcel_ts = p.parcel_timeseries["data"]
 
     # Orthogonalisation
-    if orthogonalisation not in [None, "symmetric"]:
+    if orthogonalisation not in [None, "symmetric", "none", "None"]:
         raise NotImplementedError(orthogonalisation)
 
     if orthogonalisation == "symmetric":
         logger.info(f"{orthogonalisation} orthogonalisation")
-        parcel_ts = parcellation.symmetric_orthogonalise(
-            parcel_ts, maintain_magnitudes=True
+        parcel_data = parcellation.symmetric_orthogonalise(
+            parcel_data, maintain_magnitudes=True
         )
 
+    # Create and save mne raw object for the parcellated data
+    parc_fif_file = src_dir / subject / "rhino/parc-raw.fif"
+    parc_raw = parcellation.convert2mne_raw(parcel_data.T, data)
+    parc_raw.save(parc_fif_file, overwrite=True)
+
     # Save parcellated data
     parc_data_file = src_dir / subject / "rhino/parc.npy"
     logger.info(f"saving {parc_data_file}")
-    np.save(parc_data_file, parcel_ts.T)
+    np.save(parc_data_file, parcel_data.T)
 
     # Save plots
     parcellation.plot_correlation(
-        parcel_ts,
-        filename=f"{src_dir}/{subject}/rhino/parc_corr.png",
-        logger=logger,
+        parcel_data, filename=f"{src_dir}/{subject}/rhino/parc_corr.png",
     )
 
     # Save info for the report
+    n_parcels = parcel_data.shape[0]
+    n_samples = parcel_data.shape[1]
+    if parcel_data.ndim == 3:
+        n_epochs = parcel_data.shape[2]
+    else:
+        n_epochs = None
     src_report.add_to_data(
         f"{src_dir}/{subject}/report_data.pkl",
         {
@@ -591,8 +691,9 @@ def beamform_and_parcellate(
             "method": method,
             "orthogonalisation": orthogonalisation,
             "parc_data_file": str(parc_data_file),
-            "n_samples": parcel_ts.shape[1],
-            "n_parcels": parcel_ts.shape[0],
+            "n_samples": n_samples,
+            "n_parcels": n_parcels,
+            "n_epochs": n_epochs,
             "parc_corr_plot": f"{src_dir}/{subject}/rhino/parc_corr.png",
         }
     )
@@ -659,7 +760,7 @@ def fix_sign_ambiguity(
     subject,
     preproc_file,
     smri_file,
-    logger,
+    epoch_file,
     template,
     n_embeddings,
     standardize,
@@ -679,8 +780,8 @@ def fix_sign_ambiguity(
         Path to the preprocessed fif file.
     smri_file : str
         Path to the T1 weighted structural MRI file to use in source reconstruction.
-    logger : logging.getLogger
-        Logger.
+    epoch_file : str
+        Path to epoched preprocessed fif file.
     template : str
         Template subject.
     n_embeddings : int
@@ -712,11 +813,17 @@ def fix_sign_ambiguity(
 
     # Find the channels to flip
     flips, metrics = sign_flipping.find_flips(
-        cov, template_cov, n_embeddings, n_init, n_iter, max_flips, logger
+        cov,
+        template_cov,
+        n_embeddings,
+        n_init,
+        n_iter,
+        max_flips,
+        use_tqdm=False,
     )
 
     # Apply flips to the parcellated data
-    sign_flipping.apply_flips(src_dir, subject, flips, logger)
+    sign_flipping.apply_flips(src_dir, subject, flips)
 
     # Save info for the report
     src_report.add_to_data(
