@@ -8,4 +8,4 @@ from .spmio import SPMMEEG  # noqa: F401, F403
 from .parallel import dask_parallel_bag  # noqa: F401, F403
 from .simulate import *  # noqa: F401, F403
 from .opm import *  # noqa: F401, F403
-from .package import soft_import
+from .package import soft_import, run_package_tests  # noqa: F401, F403
